from classes.spi_class import spi
from classes.LCD_display import LCD
from database.Database import Database
from flask import Flask, jsonify, request
from flask_socketio import SocketIO
from flask_cors import CORS
from RPi import GPIO
from mfrc522 import SimpleMFRC522
from RPLCD import CharLCD
from subprocess import check_output
import time, math


GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)


lcd1 = CharLCD(cols=16,rows=16,pin_rs=21,pin_e=20, pins_data=[16,12,24,23],numbering_mode=GPIO.BCM)
lcd = LCD([23,24,12,16],21,20,True)

# lcd.command(0x0f)
# lcd.command(0x01)
# lcd.clear()
# lcd.cursor_pos = (0,3)
# lcd.write_string('lcd test')

lcd.reset_lcd()


app = Flask(__name__)
CORS(app)
socketio = SocketIO(app)
db = Database(app,"mct","mct","cocktails_db")
mcp = spi(par_device=1)
rfid = SimpleMFRC522()
# lcd = LCD([23,24,12,16],21,20,True)
ultrasone = [{"trigger": 17,"echo": 27},{"trigger": 22,"echo": 5},{"trigger": 6,"echo": 13},{"trigger": 19,"echo": 26}]
pumpGPIO = {"1": 2, "2": 3, "3": 14, "4": 15}
ips = check_output(['hostname','--all-ip-addresses'])
ips = ips.decode('utf-8')

GPIO.setup(2,GPIO.OUT)
GPIO.setup(3,GPIO.OUT)
GPIO.setup(14,GPIO.OUT)
GPIO.setup(15,GPIO.OUT)

GPIO.output(2,GPIO.HIGH)
GPIO.output(3,GPIO.HIGH)
GPIO.output(14,GPIO.HIGH)
GPIO.output(15,GPIO.HIGH)

# lcd.send_msg('print')


def homescreen():
    lcd.reset_lcd()
    lcd.send_msg("IP:{0}".format(ips[13:][:13]))


def readUltraSone(sensor):
    GPIO.setup(sensor["trigger"],GPIO.OUT)
    GPIO.setup(sensor["echo"],GPIO.IN)

    GPIO.output(sensor["trigger"], True)
    time.sleep(0.00001)
    GPIO.output(sensor["trigger"], False)

    StartTime = time.time()
    StopTime = time.time()

    # save StartTime
    while GPIO.input(sensor["echo"]) == 0:
        StartTime = time.time()

    # save time of arrival
    while GPIO.input(sensor["echo"]) == 1:
        StopTime = time.time()

    TimeElapsed = StopTime - StartTime
    # multiply with the sonic speed (34300 cm/s)
    # and divide by 2, because there and back
    distance = (TimeElapsed * 34300) / 2

    return distance


def calculatetemp(value):
    thermistornominal = 10000
    temperaturenominal = 23
    bcoefficient = 3470

    tempval = (1023 / value) - 1
    tempval = 10000 / tempval

    steinhart = tempval / thermistornominal
    steinhart = math.log(steinhart)
    steinhart /= bcoefficient
    steinhart += 1.0/(temperaturenominal + 273.15)
    steinhart = 1.0 / steinhart
    steinhart -= 273.15
    tempval = steinhart
    return tempval


def calculatepercentage(waarde):
    waarde -= 3
    waarde /= 13
    waarde *= 100
    waarde = 100 - waarde
    print(waarde)
    return waarde


def readrfid():
    # lcd.clear()
    # lcd.write_string('place card below')
    socketio.emit('placecard', {'placed': 0, 'card': 0})
    lcd.reset_lcd()
    lcd.send_msg('place card below')
    id, text = rfid.read()
    lcd.reset_lcd()
    lcd.send_msg('card reconized')
    return id


def convertamount(amount):
    amount /= 100
    amount *= 90
    return amount


def activatepump(pumpnmbr,amount):
    pin = pumpGPIO[str(pumpnmbr)]
    pouringtime = convertamount(amount)
    print("pin = {0}".format(pin))
    print("time sec = {0}".format(pouringtime))
    GPIO.output(pin, GPIO.LOW)
    time.sleep(pouringtime)
    GPIO.output(pin,GPIO.HIGH)



@app.route('/')
def hallo():
    return "Server is running"


@app.route('/cocktails',methods=['GET','POST'])
def cocktails():
    if request.method == 'GET':
        data = db.get_data('SELECT * FROM cocktails')
        return jsonify(data)
    elif request.method == 'POST':
        js = request.get_json()
        # check if exists/ alraidy in database
        ifexists = db.get_data('SELECT COUNT(*) FROM cocktails WHERE Name = %s', [js['name']])
        if ifexists[0]['COUNT(*)'] == 0:
            # if not exists
            cocktail = db.set_data('INSERT INTO cocktails (Name,Price) VALUES (%s,%s)', [js['name'], js['price']])
            print(cocktail)
            if js['ingredients'][0]['bottleid'] != 0:
                ingredient1 = db.set_data(
                    'INSERT INTO bottlescocktails (Bottles_ID,Cocktails_ID,Amount_ml) VALUES (%s,%s,%s)',
                    [js['ingredients'][0]['bottleid'], cocktail, js['ingredients'][0]['amount']])
            if js['ingredients'][1]['bottleid'] != 0:
                ingredient2 = db.set_data(
                    'INSERT INTO bottlescocktails (Bottles_ID,Cocktails_ID,Amount_ml) VALUES (%s,%s,%s)',
                    [js['ingredients'][1]['bottleid'], cocktail, js['ingredients'][1]['amount']])
            if js['ingredients'][2]['bottleid'] != 0:
                ingredient3 = db.set_data(
                    'INSERT INTO bottlescocktails (Bottles_ID,Cocktails_ID,Amount_ml) VALUES (%s,%s,%s)',
                    [js['ingredients'][2]['bottleid'], cocktail, js['ingredients'][2]['amount']])
            if js['ingredients'][3]['bottleid'] != 0:
                ingredient2 = db.set_data(
                    'INSERT INTO bottlescocktails (Bottles_ID,Cocktails_ID,Amount_ml) VALUES (%s,%s,%s)',
                    [js['ingredients'][3]['bottleid'], cocktail, js['ingredients'][3]['amount']])

            return jsonify(cocktail), 201
        else:
            return 'alraidy exists'


@app.route('/delcocktail/<id>',methods=['DELETE'])
def delcocktail(id):
    ingredients = db.delete_data('DELETE FROM bottlescocktails WHERE Cocktails_ID = %s',id)
    orders = db.delete_data('DELETE FROM orders WHERE Cocktails_ID=%s',id)
    cocktail = db.delete_data('DELETE FROM cocktails WHERE cocktailID = %s',id)
    return jsonify(cocktail), 201


@app.route('/bottles',methods=['POST','GET'])
def connected_bottles():
    # Gets all bottles from database
    if request.method == 'GET':
        data = db.get_data('SELECT * FROM bottles')
        return jsonify(data)

    # inserts new bottles into database, and checks if there is a pumpnmbr
    # if there is a pump number, then it removes pumpnmbr of the previous bottle connected to the pump
    elif request.method == 'POST':
        js = request.get_json()
        ifexists = db.get_data('SELECT count(*) FROM bottles WHERE Name = %s',js['name'])
        print(js)
        # check if name alraidy in database (Need to make)
        if ifexists[0]['count(*)'] == 1 :
            if js['pump_nmbr'] == '0':
                changebottle = db.set_data('UPDATE bottles SET Pump_nmbr = NULL, Percentage_alcohol = %s WHERE Name = %s',[js['percentage'],js['name']])
                return jsonify(changebottle), 201
            elif js['pump_nmbr'] != '0':
                changepump = db.set_data('UPDATE bottles SET Pump_nmbr = NULL WHERE Pump_nmbr = %s',[js['pump_nmbr']])
                changebottle = db.set_data('UPDATE bottles SET Pump_nmbr = %s, Percentage_alcohol = %s WHERE Name = %s',[js['pump_nmbr'],js['percentage'],js['name']])
                return jsonify(changebottle), 201
        elif ifexists[0]['count(*)'] == 0:
            if js['pump_nmbr'] == '0':
                changebottle = db.set_data('INSERT INTO bottles (Name,Pump_nmbr,Percentage_alcohol) VALUES (%s,NULL,%s)',[js['name'],js['percentage']])
                return jsonify(changebottle), 201
            elif js['pump_nmbr'] != '0':
                changepump = db.set_data('UPDATE bottles SET Pump_nmbr = NULL WHERE Pump_nmbr = %s',[js['pump_nmbr']])
                changebottle = db.set_data('INSERT INTO bottles (Name,Pump_nmbr,Percentage_alcohol) VALUES (%s,%s,%s)',[js['name'],js['pump_nmbr'],js['percentage']])
                return jsonify(changebottle), 201


@app.route('/delbottles/<id>',methods=['DELETE'])
def del_bottles(id):
    inrecipe = db.get_data('SELECT count(*) FROM bottlescocktails WHERE Bottles_ID = %s',id)
    if inrecipe[0]['count(*)'] >= 1:
        return 'bottle is linked to cocktail'

    elif inrecipe[0]['count(*)'] == 0:
        delbottle = db.delete_data('DELETE FROM bottles WHERE bottlesID = %s',id)
        return jsonify(delbottle), 201


@app.route('/adduser',methods=['POST'])
def adduser():
    # adds user with rfid
    if request.method == 'POST':
        js = request.get_json()
        print(js)
        rfid_scan = readrfid()
        socketio.emit('placecard', {'placed': 1, 'rfid': rfid_scan})
        ifexists = db.get_data('SELECT COUNT(*) FROM users WHERE RFID = %s',rfid_scan)
        print(ifexists)
        # checks if the rfid alraidy exists
        if ifexists[0]['COUNT(*)'] == 1:
            print('rfid exists')
            homescreen()
            return jsonify([{'data' : 0}])
        elif ifexists[0]['COUNT(*)'] == 0:
            print('rfid doesnt exist')
            new_user = db.set_data('INSERT INTO users (Name, RFID) VALUES (%s,%s)',[js['Name'],rfid_scan])
            homescreen()
            return jsonify([{'data' : 1}]),201


@app.route('/order/<id>')
def orders(id):
    order = db.get_data('SELECT c.name as cocktailname,Datetime FROM orders LEFT JOIN cocktails as c ON orders.Cocktails_ID=c.cocktailID WHERE Users_ID = %s',id)
    return jsonify(order), 200


@app.route('/users')
def user():
    allusers = db.get_data('SELECT * FROM users')
    return jsonify(allusers), 200


@socketio.on("connect")
def connecting():
    socketio.emit("connected")
    print("Connection with client established")


@socketio.on("readrfid")
def readcard():
    # rfid test
    rfid = readrfid()
    socketio.emit('placecard',{'placed' : 1, 'rfid' : rfid})


@socketio.on("getvalues")
def getvalues():
    ultrasone0 = math.floor(calculatepercentage(readUltraSone(ultrasone[0])))
    ultrasone1 = math.floor(calculatepercentage(readUltraSone(ultrasone[1])))
    ultrasone2 = math.floor(calculatepercentage(readUltraSone(ultrasone[2])))
    ultrasone3 = math.floor(calculatepercentage(readUltraSone(ultrasone[3])))

    temp0 = math.floor(calculatetemp(mcp.readChannel(0)))
    temp1 = math.floor(calculatetemp(mcp.readChannel(1)))
    temp2 = math.floor(calculatetemp(mcp.readChannel(2)))
    temp3 = math.floor(calculatetemp(mcp.readChannel(3)))

    namebottle1 = db.get_data('SELECT name FROM bottles WHERE Pump_nmbr = 1')
    namebottle2 = db.get_data('SELECT name FROM bottles WHERE Pump_nmbr = 2')
    namebottle3 = db.get_data('SELECT name FROM bottles WHERE Pump_nmbr = 3')
    namebottle4 = db.get_data('SELECT name FROM bottles WHERE Pump_nmbr = 4')


    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[1,math.floor(readUltraSone(ultrasone[0]))])
    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[2,math.floor(readUltraSone(ultrasone[1]))])
    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[3,math.floor(readUltraSone(ultrasone[2]))])
    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[4,math.floor(readUltraSone(ultrasone[3]))])
    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[5,temp0])
    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[6,temp0])
    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[7,temp0])
    db.set_data("INSERT INTO history (Sensors_ID,value,datetime) VALUES (%s,%s,NOW())",[8,temp0])

    socketio.emit("values",[{"bottle": namebottle1[0]['name'], "ultrasonic" : ultrasone0, "temperature": temp0},{"bottle": namebottle2[0]['name'],"ultrasonic" : ultrasone1, "temperature": temp1},{"bottle": namebottle3[0]['name'],"ultrasonic" : ultrasone2, "temperature": temp2},{"bottle": namebottle4[0]['name'],"ultrasonic" : ultrasone3, "temperature": temp3}])


@socketio.on("pourcocktails")
def pourcocktail(id):
    print(id)
    countingredients = db.get_data('SELECT COUNT(*) FROM bottlescocktails WHERE cocktails_id = %s',[id])
    ifpossible = db.get_data('SELECT COUNT(B.pump_nmbr) FROM bottlescocktails AS BC JOIN bottles AS B ON BC.bottles_id = B.bottlesid WHERE cocktails_id = %s',[id])

    # print(countingredients)
    # print(ifpossible)
    # checks if the right bottles are connected to the bar
    if countingredients[0]['COUNT(*)'] == ifpossible[0]['COUNT(B.pump_nmbr)']:
         print('possible')
         # scans rfid for orders in database
         user = readrfid()
        # checks if rfid in database
         userexists = db.get_data('SELECT COUNT(*) FROM users WHERE RFID = %s',[user])
         socketio.emit('placecard', {'placed': 1, 'rfid': user})
         if userexists[0]['COUNT(*)'] == 1:
             userid = db.get_data('SELECT usersID FROM users WHERE RFID = %s',[user])
             order = db.set_data('INSERT INTO orders (Cocktails_ID,Users_ID,Datetime) VALUES (%s,%s,NOW())',[id,userid[0]['usersID']])
             ingredients = db.get_data('SELECT B.Pump_nmbr,BC.Amount_ml FROM bottlescocktails AS BC JOIN bottles AS B ON BC.Bottles_ID = B.bottlesID  WHERE cocktails_id = %s',id)
             print(ingredients)
             lcd.reset_lcd()
             lcd.send_msg('pouring ...')
             # sends what pumps should go for what time
             for i in range(0,len(ingredients)):
                 activatepump(ingredients[i]['Pump_nmbr'],ingredients[i]['Amount_ml'])
             homescreen()

         elif userexists[0]['COUNT(*)'] == 0:
             lcd.reset_lcd()
             lcd.send_msg('user not')
             lcd.second_row()
             lcd.send_msg('known')
             return 'user isnt in database'

    elif countingredients[0]['COUNT(*)'] != ifpossible[0]['COUNT(B.pump_nmbr)']:
         print('missing ingredients')


homescreen()



if __name__ == '__main__':
    socketio.run(app)





