from smbus import SMBus

class ADXL345:
    def __init__(self, par_i2c_address=0x53, range=2, full_resolution=True):
        self.i2c_address = par_i2c_address
        self.range = range
        self.full_resolution = full_resolution
        self.i2c = SMBus()

    def setup(self):
        self.i2c.open(1)
        self.i2c.write_byte_data(self.i2c_address, 0x2D, 0x08)  # Measure bit hog plaatsen in register 0x2D
        if self.full_resolution:
            self.i2c.write_byte_data(self.i2c_address, 0x31, 0x08)  # Full-RES bit hoog plaatsen op registeer 0x31
        else:
            if self.range == 2:
                self.i2c.write_byte_data(self.i2c_address, 0x31, 0x00)  # 2g resolutie
            elif self.range == 4:
                self.i2c.write_byte_data(self.i2c_address, 0x31, 0x01)  # 4g resolutie
            elif self.range == 8:
                self.i2c.write_byte_data(self.i2c_address, 0x31, 0x02)  # 8g resolutie
            elif self.range == 16:
                self.i2c.write_byte_data(self.i2c_address, 0x31, 0x03)  # 16g resolutie

    def __read_data(self):
        bytes_list = self.i2c.read_i2c_block_data(self.i2c_address, 0x32, 6)  # Bytes_list opvragen met waardes van 3 axissen in
        # Eerste 2 bytes opslaan voor x-as
        bytes_x_as = bytes_list[1] << 8
        bytes_x_as |= bytes_list[0]
        # Derde en vierde byte opslaan voor y-as
        bytes_y_as = bytes_list[3] << 8
        bytes_y_as |= bytes_list[2]
        # Laatste 2 bytes opslaan voor z-as
        bytes_z_as = bytes_list[5] << 8
        bytes_z_as |= bytes_list[4]
        return bytes_x_as, bytes_y_as, bytes_z_as

    @staticmethod
    def bits_to_int(bits):
        if 0x8000 & bits:
            negatief_getal = bits ^ 0xFFFF
            negatief_getal += 1
            return negatief_getal * (-1)
        else:
            return bits

    def get_acceleration(self):
        x, y, z, = ADXL345.__read_data(self)
        x = self.bits_to_int(x)
        y = self.bits_to_int(y)
        z = self.bits_to_int(z)
        # Geeft G krachten
        if self.full_resolution:
            return (x*4)/1000, (y*4)/1000, (z*4)/1000
        else:
            return (x*4)/(1000/(self.range/2)), (y*4)/(1000/(self.range/2)), (z*4)/(1000/(self.range/2))
